@@ -20,8 +20,8 @@ from biosimulators_utils.simulator.data_model import AlgorithmSubstitutionPolicy
 from biosimulators_utils.simulator.exceptions import AlgorithmDoesNotSupportModelFeatureException
 from biosimulators_utils.simulator.utils import get_algorithm_substitution_policy
 from biosimulators_utils.simulator.warnings import AlgorithmSubstitutedWarning
+from biosimulators_utils.utils.core import raise_errors_warnings
 import functools
-import numpy
 import os
 cwd = os.getcwd()  # because PySCeS changes the working directory
 import pysces  # noqa: E402
@@ -80,13 +80,23 @@ def exec_sed_task(task, variables, log=None):
     '''
     log = log or TaskLog()
 
-    validation.validate_task(task)
-    validation.validate_model_language(task.model.language, ModelLanguage.SBML)
-    validation.validate_model_change_types(task.model.changes, ())
-    validation.validate_model_changes(task.model.changes)
-    validation.validate_simulation_type(task.simulation, (UniformTimeCourseSimulation, ))
-    validation.validate_uniform_time_course_simulation(task.simulation)
-    validation.validate_data_generator_variables(variables)
+    model = task.model
+    sim = task.simulation
+
+    raise_errors_warnings(validation.validate_task(task),
+                          error_summary='Task `{}` is invalid.'.format(task.id))
+    raise_errors_warnings(validation.validate_model_language(task.model.language, ModelLanguage.SBML),
+                          error_summary='Language for model `{}` is not supported.'.format(model.id))
+    raise_errors_warnings(validation.validate_model_change_types(task.model.changes, ()),
+                          error_summary='Changes for model `{}` are not supported.'.format(model.id))
+    raise_errors_warnings(validation.validate_model_changes(task.model),
+                          error_summary='Changes for model `{}` are invalid.'.format(model.id))
+    raise_errors_warnings(validation.validate_simulation_type(task.simulation, (UniformTimeCourseSimulation, )),
+                          error_summary='{} `{}` is not supported.'.format(sim.__class__.__name__, sim.id))
+    raise_errors_warnings(validation.validate_simulation(task.simulation),
+                          error_summary='Simulation `{}` is invalid.'.format(sim.id))
+    raise_errors_warnings(validation.validate_data_generator_variables(variables),
+                          error_summary='Data generator variables for task `{}` are invalid.'.format(task.id))
     target_x_paths_to_sbml_ids = validation.validate_variable_xpaths(variables, task.model.source, attr='id')
 
     # Get the current working directory because PySCeS opaquely changes it
@@ -149,6 +159,9 @@ def exec_sed_task(task, variables, log=None):
         else:
             raise AlgorithmDoesNotSupportModelFeatureException('LSODA cannot execute the simulation because the model has events')
 
+    if model.mode_integrator == 'CVODE':
+        model.__settings__['cvode_return_event_timepoints'] = False
+
     # setup time course
     model.sim_start = sim.initial_time
     model.sim_end = sim.output_end_time
@@ -171,22 +184,11 @@ def exec_sed_task(task, variables, log=None):
     results, labels = model.data_sim.getAllSimData(lbls=True)
     labels = {label: i_label for i_label, label in enumerate(labels)}
 
-    desired_time_points = numpy.linspace(sim.output_start_time, sim.output_end_time, sim.number_of_points + 1)
-    saved_time_points = model.data_sim.getTime()
-    time_step = (sim.output_end_time - sim.output_start_time) / sim.number_of_points
-    diff_saved_time_points = numpy.concatenate((numpy.diff(saved_time_points), numpy.array([time_step])))
-    i_times = []
-    eps = numpy.finfo(saved_time_points.dtype).eps * 2
-    for time in desired_time_points:
-        i_time = numpy.where(numpy.logical_and(saved_time_points >= time - eps, diff_saved_time_points > 0))[0][0]
-        i_times.append(i_time)
-    i_times = numpy.array(i_times)
-
     for variable in variables:
         if variable.symbol:
             if variable.symbol == Symbol.time:
                 i_result = labels['Time']
-                variable_results[variable.id] = results[:, i_result][i_times]
+                variable_results[variable.id] = results[:, i_result][-(sim.number_of_points + 1):]
             else:
                 unpredicted_symbols.append(variable.symbol)
 
@@ -196,7 +198,7 @@ def exec_sed_task(task, variables, log=None):
             if i_result is None:
                 unpredicted_targets.append(variable.target)
             else:
-                variable_results[variable.id] = results[:, i_result][i_times]
+                variable_results[variable.id] = results[:, i_result][-(sim.number_of_points + 1):]
 
     if unpredicted_symbols:
         raise NotImplementedError("".join([
